@@ -39,15 +39,15 @@ void CitiesModel::setList()
     this->m_list.clear();
     auto cities = Cities::getInstance();
     for (const auto &cityId : (m_cities)) {
-        std::unique_ptr<City>city(cities->city(cityId));
+        City city = cities->city(cityId);
 
-        if(!city)
+        if(!city.isValid())
             continue;
 
         this->m_list << FMH::MODEL( {
-                                        {FMH::MODEL_KEY::COUNTRY, city->country()},
-                                        {FMH::MODEL_KEY::ID, city->id()},
-                                        {FMH::MODEL_KEY::NAME, city->name()}
+                                        {FMH::MODEL_KEY::COUNTRY, city.country()},
+                                        {FMH::MODEL_KEY::ID, city.id()},
+                                        {FMH::MODEL_KEY::NAME, city.name()}
                                     });
     }
     emit this->postListChanged();
