@@ -1,10 +1,14 @@
 #include "gallery.h"
+
 #include <QFileSystemWatcher>
 #include <QDateTime>
 #include <QDebug>
 #include <QDir>
 #include <QTimer>
 
+#include <QtConcurrent/QtConcurrent>
+#include <QFuture>
+
 #include <KI18n/KLocalizedString>
 
 #include <MauiKit/FileBrowsing/fileloader.h>
@@ -13,7 +17,15 @@
 #include <MauiKit/ImageTools/exiv2extractor.h>
 #include <MauiKit/ImageTools/textscanner.h>
 
-static QHash<QString, QString> TextInImages;
+#include "pix.h"
+
+static QHash<QString, QString> TextInImages; //[url:text] //global cache of text in images
+
+static QHash<QString, QString> GpsInImages()
+{
+    static QHash<QString, QString> value;
+    return value;
+}
 
 static FMH::MODEL picInfo(const QUrl &url)
 {
@@ -27,25 +39,11 @@ static FMH::MODEL picInfo(const QUrl &url)
         {FMH::MODEL_KEY::FORMAT, info.completeSuffix()}};
 }
 
-static FMH::MODEL picInfo2(const QUrl &url)
-{
-    const QFileInfo info(url.toLocalFile());
-    const Exiv2Extractor exiv2(url);
-
-    return FMH::MODEL{{FMH::MODEL_KEY::URL, url.toString()},
-        {FMH::MODEL_KEY::TITLE, info.fileName()},
-        {FMH::MODEL_KEY::SIZE, QString::number(info.size())},
-        {FMH::MODEL_KEY::SOURCE,QUrl::fromLocalFile(info.absoluteDir().absolutePath()).toString ()},
-        {FMH::MODEL_KEY::DATE, info.birthTime().toString(Qt::TextDate)},
-        {FMH::MODEL_KEY::MODIFIED, info.lastModified().toString(Qt::TextDate)},
-        {FMH::MODEL_KEY::CITY, exiv2.cityId()},
-        {FMH::MODEL_KEY::FORMAT, info.completeSuffix()}};
-}
-
 Gallery::Gallery(QObject *parent)
     : MauiList(parent)
     , m_fileLoader(new FMH::FileLoader(this))
     , m_watcher(new QFileSystemWatcher(this))
+    ,m_futureWatcher(nullptr)
     , m_scanTimer(new QTimer(this))
     , m_autoReload(true)
     , m_recursive(true)
@@ -53,13 +51,15 @@ Gallery::Gallery(QObject *parent)
     qDebug() << "CREATING GALLERY LIST";
     m_scanTimer->setSingleShot(true);
     m_scanTimer->setInterval(15000); //wait 15 secs after a new image is found and before the rescan
-
-    m_fileLoader->setBatchCount(500);
-    m_fileLoader->informer = m_activeGeolocationTags ? &picInfo2 : &picInfo;
 }
 
 Gallery::~Gallery()
 {
+    if(m_futureWatcher)
+    {
+        m_futureWatcher->cancel();
+        m_futureWatcher->deleteLater();
+    }
 }
 
 const FMH::MODEL_LIST &Gallery::items() const
@@ -75,7 +75,7 @@ void Gallery::setUrls(const QList<QUrl> &urls)
     //		return;
 
     this->m_urls = urls;
-    emit this->urlsChanged();
+    Q_EMIT this->urlsChanged();
 }
 
 QList<QUrl> Gallery::urls() const
@@ -89,7 +89,7 @@ void Gallery::setAutoReload(const bool &value)
         return;
 
     m_autoReload = value;
-    emit autoReloadChanged();
+    Q_EMIT autoReloadChanged();
 }
 
 bool Gallery::autoReload() const
@@ -129,6 +129,52 @@ void Gallery::scan(const QList<QUrl> &urls, const bool &recursive, const int &li
     m_fileLoader->requestPath(urls, recursive, FMStatic::FILTER_LIST[FMStatic::FILTER_TYPE::IMAGE], QDir::Files, limit);
 }
 
+void Gallery::scanGpsTags()
+{
+    auto functor = [](FMH::MODEL &item)
+    {
+        auto url = QUrl::fromUserInput(item[FMH::MODEL_KEY::URL]);
+
+        if(!url.isValid())
+            return;
+
+        QString cityId;
+
+        const auto urlId = url.toString();
+        if(GpsInImages().contains(urlId))
+        {
+            cityId = GpsInImages().value(urlId);
+        }else
+        {
+            qDebug() << "CREATING A NEW CITY";
+            const Exiv2Extractor exiv2(url);
+            cityId = exiv2.cityId();
+            GpsInImages().insert(urlId, cityId);
+        }
+
+        item[FMH::MODEL_KEY::CITY] = cityId;
+    } ;
+
+  m_futureWatcher = new QFutureWatcher<void>;
+   auto future = QtConcurrent::map(list, functor);
+   m_futureWatcher->setFuture(future);
+
+   connect(m_futureWatcher, &QFutureWatcher<void>::finished, [this]()
+   {
+       qDebug() << "FINISHED SCANNING GPS TAGS";
+
+        for(const auto &item : list)
+        {
+            if(m_activeGeolocationTags)
+            {
+                this->insertCity(item[FMH::MODEL_KEY::CITY]);
+            }
+        }
+
+        Q_EMIT citiesChanged();
+   });
+}
+
 void Gallery::insertFolder(const QUrl &path)
 {
     if (!m_folders.contains(path)) {
@@ -153,13 +199,13 @@ void Gallery::setStatus(const Gallery::Status &status, const QString &error)
 {
     qDebug() << "Setting up status" << status;
     this->m_status = status;
-    emit this->statusChanged();
+    Q_EMIT this->statusChanged();
 
 
     if(error != m_error)
     {
         this->m_error = error;
-        emit this->errorChanged(m_error);
+        Q_EMIT this->errorChanged(m_error);
     }
 }
 
@@ -176,36 +222,40 @@ bool Gallery::deleteAt(const int &index)
 
     const auto index_ = index;
 
-    emit this->preItemRemoved(index_);
+    Q_EMIT this->preItemRemoved(index_);
     auto item = this->list.takeAt(index_);
     FMStatic::removeFiles({item[FMH::MODEL_KEY::URL]});
-    emit this->postItemRemoved();
+    Q_EMIT this->postItemRemoved();
 
     return true;
 }
 
 void Gallery::append(const QVariantMap &pic)
 {
-    emit this->preItemAppended();
+    Q_EMIT this->preItemAppended();
     this->list << FMH::toModel(pic);
-    emit this->postItemAppended();
+    Q_EMIT this->postItemAppended();
 }
 
 void Gallery::append(const QString &url)
 {
-    emit this->preItemAppended();
+    Q_EMIT this->preItemAppended();
     this->list << picInfo(QUrl::fromUserInput(url));
-    emit this->postItemAppended();
+    Q_EMIT this->postItemAppended();
 }
 
 void Gallery::clear()
 {
-    emit this->preListChanged();
+    Q_EMIT this->preListChanged();
     this->list.clear();
-    emit this->postListChanged();
+    Q_EMIT this->postListChanged();
 
     this->m_folders.clear();
-    emit foldersChanged();
+    this->m_cities.clear();
+
+    Q_EMIT citiesChanged();
+    Q_EMIT foldersChanged();
+    Q_EMIT filesChanged();
 }
 
 void Gallery::rescan()
@@ -225,7 +275,7 @@ void Gallery::setRecursive(bool recursive)
         return;
 
     m_recursive = recursive;
-    emit recursiveChanged(m_recursive);
+    Q_EMIT recursiveChanged(m_recursive);
 }
 
 void Gallery::setlimit(int limit)
@@ -234,7 +284,7 @@ void Gallery::setlimit(int limit)
         return;
 
     m_limit = limit;
-    emit limitChanged(m_limit);
+    Q_EMIT limitChanged(m_limit);
 }
 
 int Gallery::indexOfName(const QString &query)
@@ -255,7 +305,7 @@ void Gallery::setActiveGeolocationTags(bool activeGeolocationTags)
         return;
 
     m_activeGeolocationTags = activeGeolocationTags;
-    emit activeGeolocationTagsChanged(m_activeGeolocationTags);
+    Q_EMIT activeGeolocationTagsChanged(m_activeGeolocationTags);
 }
 
 void Gallery::componentComplete()
@@ -263,9 +313,13 @@ void Gallery::componentComplete()
     connect(m_fileLoader, &FMH::FileLoader::finished, [this](FMH::MODEL_LIST items) {
         Q_UNUSED(items)
 
-        emit this->filesChanged();
-        emit this->citiesChanged();
-        emit this->foldersChanged();
+        Q_EMIT this->filesChanged();
+        Q_EMIT this->foldersChanged();
+
+        if(m_activeGeolocationTags)
+        {
+            scanGpsTags();
+        }
 
         this->setStatus(Status::Ready);
     });
@@ -276,18 +330,14 @@ void Gallery::componentComplete()
         if (items.isEmpty())
             return;
 
-        emit preItemsAppended(items.size());
+        Q_EMIT preItemsAppended(items.size());
         this->list << items;
-        emit postItemAppended();
-        emit this->countChanged();
+        Q_EMIT postItemAppended();
+        Q_EMIT this->countChanged();
     });
 
     connect(m_fileLoader, &FMH::FileLoader::itemReady, [this](FMH::MODEL item) {
-        this->insertFolder(item[FMH::MODEL_KEY::SOURCE]);
-        if(m_activeGeolocationTags)
-        {
-            this->insertCity(item[FMH::MODEL_KEY::CITY]);
-        }
+        this->insertFolder(item[FMH::MODEL_KEY::SOURCE]);       
     });
 
     connect(m_watcher, &QFileSystemWatcher::directoryChanged, [this](QString dir) {
@@ -302,14 +352,15 @@ void Gallery::componentComplete()
     connect(this, &Gallery::urlsChanged, this, &Gallery::rescan);
     connect(this, &Gallery::activeGeolocationTagsChanged, [this](bool state)
     {
-        m_fileLoader->informer = state ? &picInfo2 : &picInfo;
-
         if(state)
         {
-            this->rescan();
+            this->scanGpsTags(); //TODO change to scanGpsTags
         }
     });
 
+    m_fileLoader->setBatchCount(500);
+    m_fileLoader->informer = &picInfo;
+
     this->load();
 }
 
